#version 440 core

const int MAX_JOINTS = 20;
const int MAX_WEIGHTS = 3;

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;
layout(location = 3) in ivec3 jointIndeces;
layout(location = 4) in vec3 weights;

out VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

uniform mat4 jointTransforms[MAX_JOINTS];
uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;
uniform mat3 normalMatrix;

void main() {
	vec4 totalPosePosition = vec4(0.0);
	vec4 totalNormal = vec4(0.0);

	for(int i = 0; i < MAX_WEIGHTS; i++){
		mat4 jointTransform = jointTransforms[jointIndeces[i]];
		vec4 posePosition = jointTransform * vec4(position, 1);

		totalPosePosition += posePosition * weights[i];
		
		vec4 wordNormal = jointTransform * vec4(normal, 0.0);
		totalNormal += wordNormal * weights[i];
	}
	
	totalPosePosition = modelMatrix * totalPosePosition;

	vert.normal_world = normalMatrix * vec3(totalNormal);
	vert.uv = uv;
	vert.position_world = totalPosePosition.xyz;
	gl_Position = viewProjMatrix * totalPosePosition;
}