#version 440 core
/*
* Copyright 2010 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

layout (location = 0) out vec4 color;
layout (location = 1) out vec4 brightness;

uniform vec3 camera_world;

uniform vec4 objCol;

uniform vec4 materialCoefficients; // x = ambient, y = diffuse, z = specular, w = specularAlpha
uniform sampler2D texture_diffuse1;

/*uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;*/

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
	float brightness;
} pointL;


vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z) *  (1.0f + pointL.brightness/10);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	vec4 texColor = texture(texture_diffuse1, vert.uv).rgba;
	
	vec4 eitherColor = objCol + texColor;
	color = eitherColor * pointL.brightness * materialCoefficients.x ; // ambient
	
	// add directional light contribution
	//color.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, materialCoefficients.w, false, vec3(0));
			
	// add point light contribution
	color.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * eitherColor.rgb, materialCoefficients.y, pointL.color, materialCoefficients.z, materialCoefficients.w, true, pointL.attenuation);
	
	//do sth with brightness
	float b = dot(color.rgb, vec3(0.2126, 0.7152, 0.0722));
    if(b > 1)
        brightness = vec4(color.rgb, 1.0);
    else
        brightness = vec4(0.0, 0.0, 0.0, 1.0);
}

